<?xml version="1.0" encoding="UTF-8" ?>
<xsl:stylesheet version="1.0" 
        xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
        xmlns:xs="http://www.w3.org/2001/XMLSchema">
    <xsl:strip-space elements="*"/>
    <xsl:output method="text" /> 
 
 <!-- Copyright Kathleen Dollard 2002 -->
   
<!-- Main template that calls other templates for fragments -->
<xsl:template name="DerivedTable">
    &lt;System.Diagnostics.DebuggerStepThrough()&gt;  _
    Public Class <xsl:value-of select="@name"/>DataTable
      Inherits DataTable
      Implements System.Collections.IEnumerable
    <!-- This looks rather procedural, but is done this way so that each segment
         of the output (each set of methods) is created in a different template
         allowing you to quickly find the segment you wish to modify. -->
    <xsl:apply-templates select="DataColumn" mode="ColumnDeclarations" />
    <xsl:call-template name="TableConstructors" />
    <xsl:call-template name="TableCount" />
    <xsl:apply-templates select="DataColumn" mode="TableColumns" />
    <xsl:call-template name="TableItem" />
    <xsl:call-template name="TableRowEvents" />
    <xsl:call-template name="TableAddRow">
		<xsl:with-param name="Table" select="@name"/>
    </xsl:call-template> 
    <xsl:call-template name="TableFind" />
    <xsl:call-template name="TableEnumerator" />
    <xsl:call-template name="TableClone" />
    <xsl:call-template name="TableCreateInstance" />
    <xsl:call-template name="TableInitVars" />
    <xsl:call-template name="TableInitClass" />
    <xsl:call-template name="TableNewRow" />
    <xsl:call-template name="TableNewRowFromBuilder" />
    <xsl:call-template name="TableGetRowType" />
    <xsl:call-template name="TableOnRowChanged" />
    <xsl:call-template name="TableOnRowChanging" />
    <xsl:call-template name="TableOnRowDeleted" />
    <xsl:call-template name="TableOnRowDeleting" />
    <xsl:call-template name="TableRemoveRow" />
    End Class
</xsl:template>

<!-- Fragment Templates -->   
<xsl:template match="DataColumn" mode="ColumnDeclarations">
      Private column<xsl:value-of select="@name"/> As DataColumn
</xsl:template>

<xsl:template name="TableConstructors">
      Friend Sub New()
         MyBase.New("<xsl:value-of select="@name"/>")
         Me.InitClass
      End Sub

      Friend Sub New(ByVal table As DataTable)
         MyBase.New(table.TableName)
         If (table.CaseSensitive &lt;&gt; table.DataSet.CaseSensitive) Then
            Me.CaseSensitive = table.CaseSensitive
         End If
         If (table.Locale.ToString  &lt;&gt; table.DataSet.Locale.ToString) Then
            Me.Locale = table.Locale
         End If
         If (table.Namespace  &lt;&gt; table.DataSet.Namespace) Then
            Me.Namespace = table.Namespace
         End If
         Me.Prefix = table.Prefix
         Me.MinimumCapacity = table.MinimumCapacity
         Me.DisplayExpression = table.DisplayExpression
      End Sub
</xsl:template>

<xsl:template name="TableCount">
      &lt;System.ComponentModel.Browsable(false)> _
      Public ReadOnly Property Count As Integer
         Get
            Return Me.Rows.Count
         End Get
      End Property
</xsl:template>

<xsl:template match="DataColumn" mode="TableColumns">
      Friend ReadOnly Property <xsl:value-of select="@name"/>Column As DataColumn
         Get
            Return Me.column<xsl:value-of select="@name"/>
         End Get
      End Property
</xsl:template>

<xsl:template name="TableItem">
      Public Default ReadOnly Property Item(ByVal index As Integer) As <xsl:value-of select="@name"/>Row
         Get
            Return CType(Me.Rows(index), <xsl:value-of select="@name"/>Row)
         End Get
      End Property
</xsl:template>

<xsl:template name="TableRowEvents">
      Public Event <xsl:value-of select="@name"/>RowChanged As <xsl:value-of select="@name"/>RowChangeEventHandler
      
      Public Event <xsl:value-of select="@name"/>RowChanging As <xsl:value-of select="@name"/>RowChangeEventHandler
      
      Public Event <xsl:value-of select="@name"/>RowDeleted As <xsl:value-of select="@name"/>RowChangeEventHandler
      
      Public Event <xsl:value-of select="@name"/>RowDeleting As <xsl:value-of select="@name"/>RowChangeEventHandler
</xsl:template>

<xsl:template name="TableAddRow">
	<xsl:param name="Table" />
      Public Overloads Sub Add<xsl:value-of select="@name"/>Row(ByVal row As <xsl:value-of select="@name"/>Row)
         Me.Rows.Add(row)
      End Sub

<!--        Public Overloads Function AddOrdersRow(ByVal parentCustomersRowByCustomersOrders As CustomersRow, ByVal EmployeeID As Integer, ByVal OrderDate As Date, ByVal RequiredDate As Date, ByVal ShippedDate As Date, ByVal ShipVia As Integer, ByVal Freight As Decimal, ByVal ShipName As String, ByVal ShipAddress As String, ByVal ShipCity As String, ByVal ShipRegion As String, ByVal ShipPostalCode As String, ByVal ShipCountry As String) As OrdersRow
            Dim rowOrdersRow As OrdersRow = CType(Me.NewRow,OrdersRow)
            rowOrdersRow.ItemArray = New Object() {Nothing, parentCustomersRowByCustomersOrders(0), EmployeeID, OrderDate, RequiredDate, ShippedDate, ShipVia, Freight, ShipName, ShipAddress, ShipCity, ShipRegion, ShipPostalCode, ShipCountry}
            Me.Rows.Add(rowOrdersRow)
            Return rowOrdersRow
        End Function
-->        
      Public Overloads Function Add<xsl:value-of select="@name"/>Row(<xsl:for-each select="//Relation[ChildTable=$Table]"> ByVal parent<xsl:value-of select="ParentTable"/>RowBy<xsl:value-of select="@name"/> As <xsl:value-of select="ParentTable"/>Row,</xsl:for-each>
				<xsl:for-each select="DataColumn">
				  <xsl:variable name="ColumnName" select="@name"/>
				  <xsl:choose>
				    <xsl:when test="@AutoIncrement='true'"></xsl:when>
				    <xsl:when test="/DataSet/Relation[ChildTable=$Table and ChildKeys=$ColumnName]"></xsl:when>
				    <xsl:otherwise> ByVal <xsl:value-of select="@name"/> As <xsl:value-of select="@VBType"/><xsl:call-template name="CommaIfNotLast"/></xsl:otherwise></xsl:choose></xsl:for-each>) As <xsl:value-of select="@name"/>Row
         Dim row<xsl:value-of select="@name"/>Row As <xsl:value-of select="@name"/>Row = CType(Me.NewRow, <xsl:value-of select="@name"/>Row)
         row<xsl:value-of select="@name"/>Row.ItemArray = New Object() {<xsl:for-each select="DataColumn">
				  <xsl:variable name="ColumnName" select="@name"/>
				  <xsl:choose>
				    <xsl:when test="@AutoIncrement='true'"><xsl:call-template name="CommaIfNotFirst"/>Nothing</xsl:when>
				    <xsl:when test="/DataSet/Relation[ChildTable=$Table and ChildKeys=$ColumnName]"><xsl:call-template name="CommaIfNotFirst"/><xsl:for-each select="/DataSet/Relation[ChildTable=$Table and ChildKeys=$ColumnName]"> parent<xsl:value-of select="ParentTable"/>RowBy<xsl:value-of select="@name"/>("<xsl:value-of select="$ColumnName"/>")</xsl:for-each></xsl:when>
				    <xsl:otherwise><xsl:call-template name="CommaIfNotFirst"/> <xsl:value-of select="@name"/></xsl:otherwise></xsl:choose></xsl:for-each>}
         Me.Rows.Add(row<xsl:value-of select="@name"/>Row)
         Return row<xsl:value-of select="@name"/>Row
      End Function
</xsl:template>

<xsl:template name="TableFind">
      <xsl:variable name="TableName" select="@name"/>
      <xsl:for-each select="/DataSet/PrimaryKey[@table=$TableName]" >
      Public Function FindBy<xsl:for-each select="field" ><xsl:value-of select="."/></xsl:for-each>(<xsl:for-each select="field" ><xsl:call-template name="CommaIfNotFirst"/> ByVal <xsl:variable name="Key" select="."/><xsl:value-of select="$Key"/> As <xsl:for-each select="//DataTable[@name=$TableName]"><xsl:for-each select="DataColumn[@name=$Key]"><xsl:value-of select="@VBType"/></xsl:for-each></xsl:for-each></xsl:for-each>) As <xsl:value-of select="$TableName"/>Row
         Return CType(Me.Rows.Find(New Object() {<xsl:for-each select="field" ><xsl:call-template name="CommaIfNotFirst"/><xsl:value-of select="."/> </xsl:for-each>}), <xsl:value-of select="$TableName"/>Row)
      End Function
      </xsl:for-each>
</xsl:template>

<xsl:template name="TableEnumerator">
      Public Function GetEnumerator() As System.Collections.IEnumerator Implements System.Collections.IEnumerable.GetEnumerator
         Return Me.Rows.GetEnumerator
      End Function
</xsl:template>

<xsl:template name="TableClone">
      Public Overrides Function Clone() As DataTable
         Dim cln As <xsl:value-of select="@name"/>DataTable = CType(MyBase.Clone, <xsl:value-of select="@name"/>DataTable)
         cln.InitVars
         Return cln
      End Function
</xsl:template>

<xsl:template name="TableCreateInstance">
      Protected Overrides Function CreateInstance() As DataTable
         Return New <xsl:value-of select="@name"/>DataTable
      End Function
</xsl:template>

<xsl:template name="TableInitVars">
      Friend Sub InitVars() <xsl:for-each select="DataColumn">
          Me.column<xsl:value-of select="@name"/> = Me.Columns("<xsl:value-of select="@origname"/>")</xsl:for-each>
      End Sub
</xsl:template>

<xsl:template name="TableInitClass">
      Private Sub InitClass()  <xsl:for-each select="DataColumn">
         Me.column<xsl:value-of select="@name"/> = New DataColumn("<xsl:value-of select="@origname"/>", GetType(System.<xsl:value-of select="@type"/>), Nothing, System.Data.MappingType.Element)
         Me.Columns.Add(Me.column<xsl:value-of select="@name"/>) </xsl:for-each>
         <xsl:variable name="TableName" select="@name"/>
         <xsl:for-each select="/DataSet/PrimaryKey[@table=$TableName]">
         Me.Constraints.Add(New UniqueConstraint("<xsl:value-of select="@name"/>", New DataColumn() {<xsl:for-each select="field"><xsl:call-template name="CommaIfNotFirst"/>Me.column<xsl:value-of select="."/></xsl:for-each>}, true ))
         <xsl:if test="count(field)=1">
         <xsl:for-each select="field">Me.column<xsl:value-of select="."/>.Unique = true
         </xsl:for-each>
         </xsl:if>
         </xsl:for-each>
         <xsl:for-each select="DataColumn[@AllowDBNulls='false']">
         Me.column<xsl:value-of select="@name"/>.AllowDBNull = false</xsl:for-each>
         <xsl:for-each select="DataColumn[@ReadOnly]">
         Me.column<xsl:value-of select="@name"/>.ReadOnly = true</xsl:for-each>
         <xsl:for-each select="DataColumn[@AutoIncrement]">
         Me.column<xsl:value-of select="@name"/>.AutoIncrement = true</xsl:for-each>
      End Sub
</xsl:template>

<xsl:template name="TableNewRow">
      Public Function New<xsl:value-of select="@name"/>Row() As <xsl:value-of select="@name"/>Row
         Return CType(Me.NewRow, <xsl:value-of select="@name"/>Row)
      End Function
</xsl:template>

<xsl:template name="TableNewRowFromBuilder">
      Protected Overrides Function NewRowFromBuilder(ByVal builder As DataRowBuilder) As DataRow
         Return New <xsl:value-of select="@name"/>Row(builder)
      End Function
</xsl:template>

<xsl:template name="TableGetRowType">
      Protected Overrides Function GetRowType() As System.Type
         Return GetType(<xsl:value-of select="@name"/>Row)
      End Function
</xsl:template>

<xsl:template name="TableOnRowChanged">
      Protected Overrides Sub OnRowChanged(ByVal e As DataRowChangeEventArgs)
         MyBase.OnRowChanged(e)
         If (Not (Me.<xsl:value-of select="@name"/>RowChangedEvent) Is Nothing) Then
            RaiseEvent <xsl:value-of select="@name"/>RowChanged(Me, New <xsl:value-of select="@name"/>RowChangeEvent(CType(e.Row, <xsl:value-of select="@name"/>Row), e.Action))
         End If
      End Sub
</xsl:template>

<xsl:template name="TableOnRowChanging">
      Protected Overrides Sub OnRowChanging(ByVal e As DataRowChangeEventArgs)
         MyBase.OnRowChanging(e)
         If (Not (Me.<xsl:value-of select="@name"/>RowChangingEvent) Is Nothing) Then
            RaiseEvent <xsl:value-of select="@name"/>RowChanging(Me, New <xsl:value-of select="@name"/>RowChangeEvent(CType(e.Row, <xsl:value-of select="@name"/>Row), e.Action))
         End If
      End Sub
</xsl:template>

<xsl:template name="TableOnRowDeleted">
      Protected Overrides Sub OnRowDeleted(ByVal e As DataRowChangeEventArgs)
         MyBase.OnRowDeleted(e)
         If (Not (Me.<xsl:value-of select="@name"/>RowDeletedEvent) Is Nothing) Then
            RaiseEvent <xsl:value-of select="@name"/>RowDeleted(Me, New <xsl:value-of select="@name"/>RowChangeEvent(CType(e.Row, <xsl:value-of select="@name"/>Row), e.Action))
         End If
      End Sub
</xsl:template>

<xsl:template name="TableOnRowDeleting">
      Protected Overrides Sub OnRowDeleting(ByVal e As DataRowChangeEventArgs)
         MyBase.OnRowDeleting(e)
         If (Not (Me.<xsl:value-of select="@name"/>RowDeletingEvent) Is Nothing) Then
            RaiseEvent <xsl:value-of select="@name"/>RowDeleting(Me, New <xsl:value-of select="@name"/>RowChangeEvent(CType(e.Row, <xsl:value-of select="@name"/>Row), e.Action))
         End If
      End Sub
</xsl:template>

<xsl:template name="TableRemoveRow">
      Public Sub Remove<xsl:value-of select="@name"/>Row(ByVal row As <xsl:value-of select="@name"/>Row)
         Me.Rows.Remove(row)
      End Sub
</xsl:template>
    
</xsl:stylesheet>
  