package training.ant.task;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;

public class GrepTask extends Task {

	private String regexp;

	private File file;

	private File output;

	public void execute() throws BuildException {
		if (regexp == null) {
			throw new BuildException("attribute regexp is missing");
		}
		if (file == null) {
			throw new BuildException("attribute file is missing");
		}

		BufferedReader br = null;
		try {
			br = new BufferedReader(new FileReader(file));
		} catch (FileNotFoundException e) {
			throw new BuildException(e);
		}

		try {

			PrintWriter out;
			if (output == null) {
				out = new PrintWriter(new OutputStreamWriter(System.out));
			} else {
				out = new PrintWriter(
						new BufferedWriter(new FileWriter(output)));
			}

			Pattern p = Pattern.compile("^.*"+regexp+".*$");
			String line = null;
			int lineNumber = 0;
			while ((line = br.readLine()) != null) {
				lineNumber++;
				Matcher m = p.matcher(line);
				if (m.matches()) {
					out.print(lineNumber);
					out.print(':');
					out.println(line);
				}
			}
			br.close();
			out.close();

		} catch (IOException e) {
			throw new BuildException(e);
		}

	}

	public void setRegexp(String regexp) {
		this.regexp = regexp;
	}

	public void setFile(String fileName) {
		file = new File(fileName);
	}

	public void setOutput(String fileName) {
		output = new File(fileName);
	}
}