/*
 * From org.netbeans.nbbuild.Repeat
 */
package training.ant.task;

import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;
import java.util.Vector;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.taskdefs.CallTarget;
import org.apache.tools.ant.taskdefs.Property;

/**
 * For each of specified property values calls target task of a specified name
 * with property set to one of these values
 * 
 * @author Radim Kubacki
 */
public class Repeat extends Task {

	private List values; // List<String>

	private String target;

	private String startdir;

	private String name;

	private int times;

	private boolean timesAttributeFlag = false;

	private int start=0;

	//
	// init
	//

	public Repeat() {
		target = null;
	}

	//
	// itself
	//

	/** Number of times to call the target. */
	public void setTimes(int i) {
		log("SET times = " + i, Project.MSG_DEBUG);
		times = i;
		timesAttributeFlag = true;
	}

	/** Number of times to call the target. */
	public void setStart(int i) {
		log("SET start = " + i, Project.MSG_DEBUG);
		start = i;
	}

	
	/** Name of property that will be set for each call. */
	public void setName(String s) {
		log("SET name = " + s, Project.MSG_DEBUG);
		name = s;
	}

	/** Comma separated list of values. */
	public void setValues(String s) {
		log("SET values = " + s, Project.MSG_DEBUG);

		if(values==null) values = new Vector();
		StringTokenizer tok = new StringTokenizer(s, ", ");
		values = new Vector();
		while (tok.hasMoreTokens()) {
			values.add(tok.nextToken().trim());
		}
	}

	/**
	 * Name of target which will be used with ant task. If not specified, owning
	 * target name is used.
	 */
	public void setTarget(String s) {
		log("SET target = " + s, Project.MSG_DEBUG);

		target = s;
	}

	/** Execute this task. */
	public void execute() throws BuildException {

		if (values != null && timesAttributeFlag) {
			throw new BuildException("You must set values or times !",
					getLocation());
		}

		if (times <= 0 && timesAttributeFlag) {
			throw new BuildException(
					"Attribute times must be strict positive !", getLocation());
		}

		if (values!=null && values.isEmpty()) {
			throw new BuildException("You must set at least one value!",
					getLocation());
		}

		if (target == null) {
			throw new BuildException("Target must be set!", getLocation());
		}

		if (timesAttributeFlag) {
			times+=start;
			for(int c=start;c<times;c++) {
				String val = Integer.toString(c);

				log("Process '" + val + "' location with '" + target
						+ "' target ...", Project.MSG_VERBOSE);

				CallTarget antCall = (CallTarget) getProject().createTask(
						"antcall");
				antCall.init();
				antCall.setLocation(getLocation());

				// ant.setDir (dir);
				antCall.setTarget(target);
				Property prop = antCall.createParam();
				prop.setName(name);
				prop.setValue(val);

				antCall.execute();
			}

		} else {

			Iterator it = values.iterator();
			while (it.hasNext()) {
				String val = (String) it.next();

				log("Process '" + val + "' location with '" + target
						+ "' target ...", Project.MSG_VERBOSE);

				CallTarget antCall = (CallTarget) getProject().createTask(
						"antcall");
				antCall.init();
				antCall.setLocation(getLocation());

				// ant.setDir (dir);
				antCall.setTarget(target);
				Property prop = antCall.createParam();
				prop.setName(name);
				prop.setValue(val);

				antCall.execute();
			}
		}
	}

}