/*
 * Copyright  2004 Didier Donsez
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 * TODO: firstrow at lastrow attributes to display only a subset of the returned rows
 */
 
 package fr.imag.adele.ant.sqltoxml;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.taskdefs.JDBCTask;

public class SqlToXmlTask extends JDBCTask {

	private static final String ROW = "row";
	private static final String ID = "id";
				
	private File tofile;
	private String text;
	private int firstid = 0;
	private int idincrement = 1;
	private String rowelement = ROW;
	private String idattribute = null;	
	private String rootelement = null;
	
	/**
	 * the execute() main method
	 */
	public void execute() throws BuildException {
		Connection cnx = getConnection();
		PreparedStatement stmt = null;
		try {
			if (tofile == null) {
				throw new BuildException("Tofile must be specified");
			}
			if (text == null) {
				throw new BuildException("Statement must be specified");
			}
			Writer writer;
			try {
				writer = new BufferedWriter(new FileWriter(tofile));
			} catch (IOException e1) {
				throw new BuildException(
					"Can't open file:" + tofile.getAbsolutePath());
			}

			String sql = text;
			stmt = cnx.prepareStatement(sql);
			if (stmt.execute()) {
				ResultSet rs = stmt.getResultSet();
				ResultSetMetaData rsmd = rs.getMetaData();
				int numcols = rsmd.getColumnCount();

				StringBuffer sb = new StringBuffer();
				if(rootelement!=null) sb.append("<").append(rowelement).append(">\n");
				int currentid=firstid;
				while (rs.next()) {
					sb.append("<").append(rowelement);
					if(idattribute!=null) sb.append(" ").append(idattribute).append("=\"").append(currentid).append("\"");
					sb.append(">\n");
					currentid += idincrement;
					for (int i = 1; i <= numcols; i++) {
						String columnLabel = rsmd.getColumnLabel(i);
						Object obj = rs.getObject(i);
						String columnValue;
						if (obj != null) {
							sb.append('<').append(columnLabel).append('>');
							sb.append(obj.toString());
							sb.append("</").append(columnLabel).append(">\n");
						} else {
							sb.append('<').append(columnLabel).append("/>\n");
						}
					}
					sb.append("</").append(rowelement).append(">\n");
				}
				if(rootelement!=null) sb.append("</").append(rowelement).append(">\n");
				log(sb.toString());
				try {
					writer.write(sb.toString());
					writer.close();
				} catch (IOException e2) {
					throw new BuildException(
						"Can't write into file:" + tofile.getAbsolutePath());
				}
			}
		} catch (SQLException e) {

		} finally {
			if (stmt != null) {
				try {
					stmt.close();
				} catch (SQLException ignore) {
				}
			}
			if (cnx != null) {
				try {
					cnx.close();
				} catch (SQLException ignore) {
				}
			}
		}
	}
	/**
	 * @param tofile the name of the generated file
	 */
	public void setTofile(File tofile) {
		this.tofile = tofile;
	}

	/**
	 * @param text	the SQL (select) statement
	 */
	public void addText(String text) {
		this.text = text;
	}
	
	/**
	 * @param i	the first value of the id attribute
	 */
	public void setFirstid(int i) {
		firstid = i;
	}

	/**
	 * @param i the increment value of the id attribute
	 */
	public void setIdincrement(int i) {
		idincrement = i;
	}

	/**
	 * @param rowelement the name of the row element (yes, row)
	 */
	public void setRowelement(String rowelement) {
		this.rowelement = rowelement;
	}

	/**
	 * @param rootelement the name of the root element (no)
	 */
	public void setRootelement(String rootelement) {
		this.rootelement = rootelement;
	}

	/**
	 * @param idattribute the name of the row id attribute (yes, id)
	 */
	public void setIdattribute(String idattribute) {
		this.idattribute = idattribute;
	}

}
